<?php
/*
 *  $Id: Config.php 378 2011-12-03 12:40:59Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Config
 *
 * The class is the common config in psx. It includes the file provided in the
 * first argument of the constructor. This file must define an array $config
 * wich is assigned to the private property $container. Here an example how to
 * create an access the config.
 * <code>
 * $config = new PSX_Config('configuration.php');
 *
 * echo $config['psx_url'];
 * </code>
 *
 * The config class also provides methods to get useful informations of the
 * current request. For more informations see the methods.
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Config
 * @version    $Revision: 378 $
 */
class PSX_Config extends ArrayIterator
{
	/**
	 * The current request method
	 *
	 * @var string
	 */
	private static $requestMethod;

	/**
	 * Indicates whether the method was overriden or not
	 *
	 * @var boolean
	 */
	private static $methodOverride = false;

	/**
	 * Contains the request header if the get_request_header methid is
	 * called
	 *
	 * @var array
	 */
	private static $headers;

	/**
	 * Contains the raw request
	 *
	 * @var string
	 */
	private static $rawInput;

	/**
	 * Contains the absolute url to the script using the psx_url from the
	 * configuration
	 *
	 * @var string
	 */
	private $self;

	/**
	 * The host of the value of psx_url
	 *
	 * @var string
	 */
	private $host;

	/**
	 * The container for the config array
	 *
	 * @var array
	 */
	private $container = array();

	public function __construct($file)
	{
		include($file);

		if(isset($config))
		{
			// assign container
			parent::__construct($config);


			// set default timezone
			$timezone = $this->offsetGet('psx_timezone');

			if(empty($timezone) || !date_default_timezone_set($timezone))
			{
				throw new PSX_Config_Exception('Invalid timezone');
			}


			// assign the host
			$url = $this->offsetGet('psx_url');

			if(!empty($url))
			{
				$parts = parse_url($url);

				if($parts !== false && isset($parts['scheme']) && isset($parts['host']))
				{
					$this->self = $parts['scheme'] . '://' . $parts['host'] . (isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '');
					$this->host = $parts['host'];
				}
				else
				{
					throw new PSX_Config_Exception('Invalid PSX url');
				}
			}
			else
			{
				throw new PSX_Config_Exception('PSX url not set');
			}
		}
		else
		{
			throw new PSX_Config_Exception('Couldnt find config in file');
		}
	}

	/**
	 * getSelf
	 *
	 * Returns the absolute url of the current requested url
	 *
	 * @return string
	 */
	public function getSelf()
	{
		return $this->self;
	}

	/**
	 * getHost
	 *
	 * Returns the host name of the url
	 *
	 * @return string
	 */
	public function getHost()
	{
		return $this->host;
	}

	/**
	 * getUrn
	 *
	 * Returns an uniqe urn for the domain. Accepts variable arguments wich
	 * are concatenated with an colon ':'
	 *
	 * @see http://www.ietf.org/rfc/rfc2141.txt
	 * @return string
	 */
	public function getUrn()
	{
		$ns   = array('psx', $this->host);
		$args = func_get_args();

		return 'urn:' . implode(':', array_merge($ns, $args));
	}

	/**
	 * getUUID
	 *
	 * Generates an Name-Based UUID where the namespace is the host of this
	 * domain
	 *
	 * @see http://www.ietf.org/rfc/rfc4122.txt
	 * @return string
	 */
	public function getUUID($name)
	{
		$namespace = $this->host;
		$hash      = sha1($namespace . $name);

		$timeLow            = substr($hash, 0, 8);
		$timeMid            = substr($hash, 8, 4);
		$timeHighVersion    = dechex(hexdec(substr($hash, 12, 4)) & 0x0FFF | 0x5000);
		$clockSeqHiReserved = dechex(hexdec(substr($hash, 16, 2)) & 077 | 0200);
		$clockSeqLow        = substr($hash, 17, 2);
		$node               = substr($hash, 18, 12);

		return $timeLow . '-' . $timeMid . '-' . $timeHighVersion . '-' . $clockSeqHiReserved . $clockSeqLow . '-' . $node;
	}

	/**
	 * buildId
	 *
	 * @return string
	 */
	public function buildId($url, DateTime $date, $id)
	{
		return 'tag:' . $this->host . ',' . $date->format('Y-m-d') . ':' . $id;
	}

	/**
	 * getRequestMethod
	 *
	 * Tries to detect the current request method. It considers the
	 * X-HTTP-METHOD-OVERRIDE header.
	 *
	 * @return string
	 */
	public static function getRequestMethod()
	{
		if(self::$requestMethod === null)
		{
			if(isset($_SERVER['REQUEST_METHOD']))
			{
				// check for X-HTTP-Method-Override
				if(isset($_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE']) && in_array($_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'], array('OPTIONS', 'GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'TRACE', 'CONNECT')))
				{
					self::$methodOverride = true;

					self::$requestMethod  = $_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'];
				}
				else
				{
					self::$methodOverride = false;

					self::$requestMethod  = $_SERVER['REQUEST_METHOD'];
				}
			}
			else
			{
				self::$requestMethod = 'GET';
			}
		}

		return self::$requestMethod;
	}

	/**
	 * isOverride
	 *
	 * Tells whether the method was overwriten by an X-HTTP-METHOD-OVERRIDE
	 * header or not
	 *
	 * @return boolean
	 */
	public static function isOverride()
	{
		return self::$methodOverride;
	}

	/**
	 * get_request_header
	 *
	 * Returns all or a specific request header. Note if you get all request
	 * headers as array all keys are lowercase since the specification says
	 * that http headers are case-insensitive so you dont have to search
	 * the complete array instead you can acces it with the key i.e.:
	 * <code>
	 * $headers = PSX_Config::getRequestHeader();
	 *
	 * echo $headers['content-type'];
	 * </code>
	 *
	 * If the parameter $key is defined it searches for the specific header
	 * and returns the value if its exists else false
	 *
	 * @param string $key
	 * @return string|array
	 */
	public static function getRequestHeader($key = false)
	{
		if(self::$headers === null)
		{
			if(function_exists('apache_request_headers'))
			{
				$headers = apache_request_headers();

				foreach($headers as $k => $v)
				{
					$k = strtolower($k);

					self::$headers[$k] = $v;
				}
			}
			else
			{
				foreach($_SERVER as $k => $v)
				{
					if(substr($k, 0, 5) == 'HTTP_')
					{
						$k = str_replace('_', '-', strtolower(substr($k, 5)));

						self::$headers[$k] = $v;
					}
				}
			}
		}

		if($key === false)
		{
			return self::$headers;
		}
		else
		{
			$key = strtolower($key);

			return isset(self::$headers[$key]) ? self::$headers[$key] : false;
		}
	}

	/**
	 * getRawInput
	 *
	 * Returns the raw input of the current request
	 *
	 * @return string
	 */
	public static function getRawInput()
	{
		if(self::$rawInput === null)
		{
			self::$rawInput = file_get_contents('php://input');
		}

		return self::$rawInput;
	}
}

