<?php
/*
 *  $Id: File.php 260 2011-03-30 20:00:01Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_File
 *
 * This class is a wrapper to the f* functions to offer them in an OOP way. It
 * is a good question when you ask yourself why to use an wrapper for the file
 * functions. The answer is that when the function changes or you want add extra
 * functionality to the functions you can add them easily in this class and you
 * dont have to rewrite your code. So it is more consistent to use this class
 * for file actions. If I cant convince you here an real life example why to use
 * the PSX_File wrapper. Lets assume that we have wrote many classes that uses
 * the f* functions and now we want log every file request. If you have used the
 * f* functions you have to add in every code where you use the fopen function a
 * log functionality. If we have used the PSX_File class you can write the log
 * functionality to the __construct  and all fopen requests will now logged
 * without a change to other code. If the function cant open a file it throws an
 * exception. Here an example howto create and write a file.
 * <code>
 * $file = new PSX_File('cache/test.txt', 'w');
 *
 * $file->write('foobar');
 *
 * $file->close();
 * </code>
 *
 * You can access all f* functions as methods in PSX_File. You can find a list
 * of all file function at the php manual
 * http://php.net/manual/ref.filesystem.php In general all functions that accept
 * at first argument an file handle can accessed as method. Here another example
 * howto read the content of a file
 * <code>
 * $file = new PSX_File('cache/test.txt', 'r');
 *
 * $buffer = '';
 *
 * while(!$file->eof())
 * {
 * 	$buffer.= $file->gets(512);
 * }
 * </code>
 *
 * Obviously it is in this case easier to call the PSX_File::getContents() function
 * but it is an example how the class works.
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   psx
 * @package    psx_file
 * @version    $Revision: 260 $
 */
class PSX_File
{
	/**
	 * The file where this class is operating on
	 *
	 * @var string
	 */
	public $file;

	/**
	 * The mode how file was opened
	 *
	 * @var string
	 */
	public $mode;

	/**
	 * The file resource handler
	 *
	 * @var resource
	 */
	public $handle;

	/**
	 * __construct
	 *
	 * Opens a new file with the fopen function. If not other specified the
	 * file is opened in a+ mode. If the fopen function fails the method
	 * throws an exception.
	 *
	 * @param string $file
	 * @param string $mode
	 * @return void
	 */
	public function __construct($file, $mode = false)
	{
		$mode = $mode === false ? 'a+' : $mode;

		$handle = fopen($file, $mode);

		if(!$handle)
		{
			throw new PSX_File_Exception('Could not open file ' . $file . ' in mode ' . $mode);
		}
		else
		{
			$this->file   = $file;
			$this->handle = $handle;
			$this->mode   = $mode;
		}
	}

	/**
	 * __call
	 *
	 * This method is the wrapper to the f* functions. It is implemented
	 * with the magic method __call. Probably it is faster to hardcode the
	 * methods but it was easier to implement ^^.
	 *
	 * @param string $name
	 * @param array $parameter
	 * @return mixed
	 */
	public function __call($name, $parameter)
	{
		$functions = array(

			'close'    => 'fclose',
			'eof'      => 'feof',
			'flush'    => 'fflush',
			'getc'     => 'fgetc',
			'getcsv'   => 'fgetcsv',
			'gets'     => 'fgets',
			'getss'    => 'fgetss',
			'lock'     => 'flock',
			'passthru' => 'fpassthru',
			'putcsv'   => 'fputcsv',
			'puts'     => 'fputs',
			'read'     => 'fread',
			'scanf'    => 'fscanf',
			'seek'     => 'fseek',
			'stat'     => 'fstat',
			'tell'     => 'ftell',
			'truncate' => 'ftruncate',
			'write'    => 'fwrite',
			'rewind'   => 'rewind',

		);

		if(array_key_exists($name, $functions))
		{
			$function  = $functions[$name];
			$parameter = array_merge(array($this->handle), $parameter);

			return call_user_func_array($function, $parameter);
		}
	}

	/**
	 * isFile
	 *
	 * A wrapper for the is_file function. Because the is_file function
	 * takes a lot of time we ... but on the otherside its much more safer.
	 * Change true to false if you want speed up your app
	 *
	 * @param string $file
	 * @return boolean
	 */
	public static function isFile($file)
	{
		if(true)
		{
			return is_file($file);
		}
		else
		{
			return true;
		}
	}

	/**
	 * getContents
	 *
	 * @param string $file
	 * @return string
	 */
	public static function getContents($file)
	{
		return file_get_contents($file);
	}
}

