<?php
/*
 *  $Id: Paging.php 282 2011-05-18 23:40:32Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Html_Paging
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Html
 * @version    $Revision: 282 $
 */
class PSX_Html_Paging implements ArrayAccess, Countable
{
	private $url;
	private $resultSet;
	private $range;

	private $nav = array();
	private $totalResults;
	private $itemsPerPage;
	private $startIndex;
	private $count;

	public $page;
	public $pages;
	public $first;
	public $prev;
	public $min;
	public $max;
	public $next;
	public $last;

	public function __construct(PSX_Url $url, PSX_Data_ResultSet $resultSet, $range = 2)
	{
		// assign values
		$this->url       = $url;
		$this->resultSet = $resultSet;
		$this->range     = $range;


		// get values
		$this->totalResults = intval($resultSet->totalResults);
		$this->itemsPerPage = intval($resultSet->itemsPerPage);
		$this->startIndex   = intval($resultSet->startIndex);
		$this->count        = $resultSet->getLength();


		// do the math
		$this->calcPages();


		// build the urls
		$this->buildNav();
	}

	public function getResultSet()
	{
		return $this->resultSet;
	}

	public function getRange()
	{
		return $this->range;
	}

	public function calcPages()
	{
		$this->pages = $this->totalResults > 0 && $this->totalResults > $this->itemsPerPage ? ceil($this->totalResults / $this->itemsPerPage) - 1 : 0;
		$this->page  = $this->startIndex >= 0 ? intval($this->startIndex / $this->itemsPerPage) : 0;
		$this->page  = $this->page < 0 ? 0 : $this->page; # lower limit
		$this->page  = $this->page > $this->pages ? $this->pages : $this->page; # upper limit

		$this->first = 0;
		$this->prev  = $this->page > 1 ? $this->page - 1 : 0;
		$this->next  = $this->page < $this->pages ? $this->page + 1 : $this->pages;
		$this->last  = $this->pages;

		$this->min   = $this->page - $this->range < 0 ? 0 : $this->page - $this->range;
		$this->max   = $this->page + $this->range > $this->pages ? $this->pages : $this->page + $this->range;
	}

	private function buildNav()
	{
		if($this->pages > 0)
		{
			$this->url->addParam('count', $this->itemsPerPage, true);

			$this->nav['page']  = $this->page + 1;
			$this->nav['pages'] = $this->pages + 1;

			$this->url->addParam('startIndex', $this->first * $this->itemsPerPage, true);

			$this->nav['first'] = strval($this->url);

			$this->url->addParam('startIndex', $this->prev * $this->itemsPerPage, true);

			$this->nav['prev'] = strval($this->url);

			$this->nav['range'] = array();

			for($i = $this->min; $i <= $this->max; $i++)
			{
				$this->url->addParam('startIndex', $i * $this->itemsPerPage, true);

				if($i == $this->page)
				{
					$this->nav['range'][] = array('name' => $i + 1, 'current' => true, 'href' => strval($this->url));
				}
				else
				{
					$this->nav['range'][] = array('name' => $i + 1, 'href' => strval($this->url));
				}
			}

			$this->url->addParam('startIndex', $this->next * $this->itemsPerPage, true);

			$this->nav['next'] = strval($this->url);

			$this->url->addParam('startIndex', $this->last * $this->itemsPerPage, true);

			$this->nav['last'] = strval($this->url);
		}
		else
		{
			$this->nav['page']  = 0;
			$this->nav['pages'] = 0;
		}
	}

	// ArrayAccess
	public function offsetGet($offset)
	{
		if(isset($this->nav[$offset]))
		{
			return $this->nav[$offset];
		}
		else
		{
			throw new PSX_Html_Paging_Exception('Invalid paging offset');
		}
	}

	public function offsetSet($offset, $value)
	{
	}

	public function offsetExists($offset)
	{
		return isset($this->nav[$offset]);
	}

	public function offsetUnset($offset)
	{
	}

	// Countable
	public function count()
	{
		return $this->pages;
	}
}


