<?php
/*
 *  $Id: Http.php 260 2011-03-30 20:00:01Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Http
 *
 * This class offers a simple way to make http requests to other websites.
 * I think the easiest way is to show you some examples how it works.
 * <code>
 * $http = new PSX_Http(new PSX_Http_Handler_Curl());
 *
 * // get request
 * $request  = new PSX_Http_GetRequest('http://127.0.0.1');
 * $response = $http->request($request);
 *
 * // post request
 * $request  = new PSX_Http_PostRequest('http://127.0.0.1', array(
 *
 * 	'id'    => 1,
 * 	'title' => 'test',
 *
 * ));
 * $response = $http->request($request);
 *
 * // put request
 * $request  = new PSX_Http_PutRequest('http://127.0.0.1', array(
 *
 * 	'id'    => 1,
 * 	'title' => 'test',
 *
 * ));
 * $response = $http->request($request);
 *
 * // delete request
 * $request  = new PSX_Http_DeleteRequest('http://127.0.0.1');
 * $response = $http->request($request);
 * </code>
 *
 * If you make an put or delete request you can optional specify with the
 * argument $override whether to use a real PUT or DELETE request or use an
 * X-HTTP-Method-Override header to simulate such an request because some
 * firewalls block an PUT or DELETE request. If you override a PUT or DELETE
 * request it becomes an POST request. The response is an object of the class
 * PSX_Http_Response. Here an example how you can use the class see the
 * PSX_Http_Response for more detailed informations.
 * <code>
 *
 * if($response->getCode() == 200)
 * {
 * 	$header = $response->getHeader();
 * 	$body   = $response->getBody();
 *
 * 	if(isset($header['content-type']) && $header['content-type'] == 'foo/bar')
 * 	{
 * 		echo $body;
 * 	}
 * }
 * </code>
 *
 * You can use the getLastError method to check whether the request was
 * successful. This is especially useful if you make https requests. I.e.
 * <code>
 * $http    = new PSX_Http(new PSX_Http_Handler_Curl());
 * $reponse = $http->request(new PSX_Http_GetRequest('https://encrypted.google.com/'));
 * $error   = $http->getLastError();
 *
 * if(empty($error))
 * {
 * 	echo 'Everything works .. ;D';
 * }
 * else
 * {
 * 	throw new PSX_Exception($error);
 * }
 * </code>
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Http
 * @version    $Revision: 260 $
 */
class PSX_Http
{
	public static $newLine = "\r\n";
	public static $codes   = array(

		200 => 'OK',
		201 => 'Created',
		202 => 'Accepted',
		203 => 'Non-Authoritative Information',
		204 => 'No Content',
		205 => 'Reset Content',
		206 => 'Partial Content',
		300 => 'Multiple Choices',
		301 => 'Moved Permanently',
		302 => 'Found',
		303 => 'See Other',
		304 => 'Not Modified',
		305 => 'Use Proxy',
		307 => 'Temporary Redirect',
		400 => 'Bad Request',
		401 => 'Unauthorized',
		402 => 'Payment Required',
		403 => 'Forbidden',
		404 => 'Not Found',
		405 => 'Method Not Allowed',
		406 => 'Not Acceptable',
		407 => 'Proxy Authentication Required',
		408 => 'Request Timeout',
		409 => 'Conflict',
		410 => 'Gone',
		411 => 'Length Required',
		412 => 'Precondition Failed',
		413 => 'Request Entity Too Large',
		414 => 'Request-URI Too Long',
		415 => 'Unsupported Media Type',
		416 => 'Requested Range Not Satisfiable',
		417 => 'Expectation Failed',
		500 => 'Internal Server Error',
		501 => 'Not Implemented',
		502 => 'Bad Gateway',
		503 => 'Service Unavailable',
		504 => 'Gateway Timeout',
		505 => 'HTTP Version Not Supported'

	);

	private $lastError;
	private $request;
	private $response;

	public function __construct(PSX_Http_HandlerInterface $handler)
	{
		$this->handler = $handler;
	}

	public function request(PSX_Http_Request $request)
	{
		$response = $this->handler->request($request);

		$this->lastError = $this->handler->getLastError();
		$this->request   = $this->handler->getRequest();
		$this->response  = $this->handler->getResponse();

		return PSX_Http_Response::convert($response);
	}

	public function getLastError()
	{
		return $this->lastError;
	}

	public function getRequest()
	{
		return $this->request;
	}

	public function getResponse()
	{
		return $this->response;
	}
}

