<?php
/*
 *  $Id: InputAbstract.php 354 2011-10-29 15:25:33Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_InputAbstract
 *
 * This class is for handling data sources wich have untrusted values i.e. GET
 * or POST values. Because this is an abstract class you cant create an instance
 * from it therefor here some examples with the PSX_Get class how to access the
 * values and how to apply filters. In the same way you can use all other
 * classes wich extends PSX_InputAbstract.
 *
 * <code>
 * $get = new PSX_Get(new PSX_Validate());
 *
 * // returns intval($_GET['foo']) or false if foo ist not set
 * $bar = $get->foo('integer');
 *
 * // foo must be an integer value >= 3 and <= 64
 * $bar = $get->foo('integer', array(new PSX_Filter_Length(3, 64)));
 *
 * // returns an md5 representation of foo if it contains only alphanumeric
 * // signs else it returns false
 * $bar = $get->foo('string', array(new PSX_Filter_Alnum(), new PSX_Filter_Md5()));
 * </code>
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_InputAbstract
 * @version    $Revision: 354 $
 */
abstract class PSX_InputAbstract implements ArrayAccess
{
	private $validate;

	public function __construct(PSX_Validate $validate)
	{
		$this->validate = $validate;
	}

	/**
	 * has
	 *
	 * @param string $key
	 * @return boolean
	 */
	abstract public function has($key);

	/**
	 * get
	 *
	 * @param string $key
	 * @return mixed
	 */
	abstract public function get($key);

	/**
	 * set
	 *
	 * @param string $key
	 * @param mixed $value
	 * @return void
	 */
	abstract public function set($key, $value);

	/**
	 * delete
	 *
	 * @param string $key
	 * @return void
	 */
	abstract public function delete($key);

	/**
	 * getContainer
	 *
	 * @return array
	 */
	abstract public function getContainer();

	/**
	 * getValidator
	 *
	 * @return psx_validate
	 */
	public function getValidator()
	{
		return $this->validate;
	}

	/**
	 * __get
	 *
	 * @param string $key
	 * @return false|string
	 */
	public function __get($key)
	{
		return $this->get($key);
	}

	/**
	 * __call
	 *
	 * @param string $key
	 * @param array $parameters
	 * @return false|integer|float|string|boolean
	 */
	public function __call($key, $parameters)
	{
		$value    = $this->get($key);
		$type     = isset($parameters[0]) ? $parameters[0] : 'string';
		$filter   = isset($parameters[1]) ? $parameters[1] : array();
		$key      = isset($parameters[2]) ? $parameters[2] : $key;
		$title    = isset($parameters[3]) ? $parameters[3] : $key;
		$required = isset($parameters[4]) ? $parameters[4] : true;
		$return   = isset($parameters[5]) ? $parameters[5] : false;

		return $this->validate->apply($value, $type, $filter, $key, $title, $required, $return);
	}

	// ArrayAccess
	public function offsetExists($offset)
	{
		return $this->has($offset);
	}

	public function offsetGet($offset)
	{
		return $this->get($offset);
	}

	public function offsetSet($offset, $value)
	{
		$this->set($offset, $value);
	}

	public function offsetUnset($offset)
	{
		$this->delete($offset);
	}
}

