<?php
/*
 *  $Id: Opengraph.php 260 2011-03-30 20:00:01Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Opengraph
 *
 * Discovers opengraph tags on an specific url. The discovery method make an GET
 * request to the specified url and tries to fetch all opengraph tags from the
 * response body. You can also specifiy wich tags should be fetched in example:
 * <code>
 * $http = new PSX_Http(new PSX_Http_Handler_Curl());
 * $og   = new PSX_Opengraph($http)
 *
 * $tags = $og->discover(new psx_type_url('http://opengraphprotocol.org/'));
 *
 * $tags = $og->discover(new psx_type_url('http://opengraphprotocol.org/'), psx_net_opengraph::TITLE | psx_net_opengraph::TYPE);
 * </code>
 *
 * The discover method returns an associative array wich has as key the meta
 * name without "og:" and "-" are replaced with "_". As value either false if
 * the tag was not found or the value of the content attribute.
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @see        http://opengraphprotocol.org
 * @category   PSX
 * @package    PSX_Opengraph
 * @version    $Revision: 260 $
 */
class PSX_Opengraph
{
	const TITLE          = 1;
	const TYPE           = 2;
	const URL            = 4;
	const IMAGE          = 8;
	const DESCRIPTION    = 16;
	const SITE_NAME      = 32;
	const LATITUDE       = 64;
	const LONGITUDE      = 128;
	const STREET_ADDRESS = 256;
	const LOCALITY       = 512;
	const REGION         = 1024;
	const POSTAL_CODE    = 2048;
	const COUNTRY_NAME   = 4096;
	const ALL            = 8192;

	private $metadata = array(

		'title'         => 'og:title',
		'type'          => 'og:type',
		'url'           => 'og:url',
		'image'         => 'og:image',
		'description'   => 'og:description',
		'siteName'      => 'og:site_name',
		'latitude'      => 'og:latitude',
		'longitude'     => 'og:longitude',
		'streetAddress' => 'og:street-address',
		'locality'      => 'og:locality',
		'region'        => 'og:region',
		'postalCode'    => 'og:postal-code',
		'countryName'   => 'og:country-name'

	);

	private $http;

	public function __construct(PSX_Http $http)
	{
		$this->http = $http;
	}

	public function discover(PSX_Url $url, $type = PSX_Opengraph::ALL)
	{
		$request   = new PSX_Http_GetRequest($url);

		$response  = $this->http->request($request);
		$lastError = $this->http->getLastError();

		if(empty($lastError))
		{
			$data   = array();
			$fields = array();
			$parser = new PSX_Html_Parse($response->getBody());

			if($type & self::TITLE)          { $fields['title']         = $this->metadata['title']; }
			if($type & self::TYPE)           { $fields['type']          = $this->metadata['type']; }
			if($type & self::URL)            { $fields['url']           = $this->metadata['url']; }
			if($type & self::IMAGE)          { $fields['image']         = $this->metadata['image']; }
			if($type & self::DESCRIPTION)    { $fields['description']   = $this->metadata['description']; }
			if($type & self::SITE_NAME)      { $fields['siteName']      = $this->metadata['siteName']; }
			if($type & self::LATITUDE)       { $fields['latitude']      = $this->metadata['latitude']; }
			if($type & self::LONGITUDE)      { $fields['longitude']     = $this->metadata['longitude']; }
			if($type & self::STREET_ADDRESS) { $fields['streetAddress'] = $this->metadata['streetAddress']; }
			if($type & self::LOCALITY)       { $fields['locality']      = $this->metadata['locality']; }
			if($type & self::REGION)         { $fields['region']        = $this->metadata['region']; }
			if($type & self::POSTAL_CODE)    { $fields['postalCode']    = $this->metadata['postalCode']; }
			if($type & self::COUNTRY_NAME)   { $fields['countryName']   = $this->metadata['countryName']; }
			if($type & self::ALL)            { $fields                  = $this->metadata; }

			$element = new PSX_Html_Parse_Element('meta');

			foreach($fields as $key => $meta)
			{
				$element->setAttributes(array('property' => $meta));

				$value = $parser->fetchAttrFromHead($element, 'content');

				$data[$key] = $value;
			}

			return $data;
		}
		else
		{
			throw new PSX_Opengraph_Exception($lastError);
		}
	}
}

