<?php
/*
 *  $Id: Select.php 371 2011-11-20 18:36:52Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Sql_Table_Select
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Sql
 * @version    $Revision: 371 $
 */
class PSX_Sql_Table_Select implements PSX_Sql_Table_SelectInterface
{
	protected $table;
	protected $sql;
	protected $condition;

	protected $joins   = array();
	protected $columns = array();
	protected $prefix;

	protected $selfColumns      = array();
	protected $availableColumns = array();

	protected $start;
	protected $count;
	protected $sortOrder;
	protected $sortBy;
	protected $groupBy;

	public function __construct(PSX_Sql_TableInterface $table, $prefix = null)
	{
		$this->table            = $table;
		$this->sql              = $table->getSql();
		$this->condition        = new PSX_Sql_Condition();

		$this->setPrefix($prefix);
	}

	public function join($type, $table, $cardinality = 'n:1', $foreignKey = null)
	{
		if($table instanceof PSX_Sql_TableInterface)
		{
		}
		else if($table instanceof PSX_Sql_Table_SelectInterface)
		{
			$table = $table->getTable();
		}
		else
		{
			throw new PSX_Sql_Table_Exception('Invalid table must be instanceof PSX_Sql_TableInterface or PSX_Sql_Table_SelectInterface');
		}

		if($table->getLastSelect() === null)
		{
			throw new PSX_Sql_Table_Exception('Nothing is selected on table ' . $table->getAlias());
		}

		$this->joins[] = new PSX_Sql_Join($type, $table, $cardinality, $foreignKey);

		$this->condition->merge($table->getLastSelect()->getCondition());

		$this->availableColumns = array_merge($this->availableColumns, $table->getLastSelect()->getAllColumns());

		return $this;
	}

	public function where($column, $operator, $value, $conjunction = 'AND')
	{
		if(!isset($this->availableColumns[$column]))
		{
			throw new PSX_Sql_Table_Exception('Invalid column');
		}

		$this->condition->add($this->availableColumns[$column], $operator, $value, $conjunction);

		return $this;
	}

	public function groupBy($column)
	{
		if(!isset($this->availableColumns[$column]))
		{
			throw new PSX_Sql_Table_Exception('Invalid column');
		}

		$this->groupBy = $this->availableColumns[$column];

		return $this;
	}

	public function orderBy($column, $sort = 0x1)
	{
		if(!isset($this->availableColumns[$column]))
		{
			throw new PSX_Sql_Table_Exception('Invalid column');
		}

		$this->sortOrder = $sort === PSX_Sql::SORT_ASC ? 'ASC' : 'DESC';
		$this->sortBy    = $column;

		return $this;
	}

	public function limit($start, $count = null)
	{
		if($count === null)
		{
			$this->start = 0;
			$this->count = (integer) $start;
		}
		else
		{
			$this->start = (integer) $start;
			$this->count = (integer) $count;
		}

		return $this;
	}

	public function getTable()
	{
		return $this->table;
	}

	public function getSql()
	{
		return $this->sql;
	}

	public function getCondition()
	{
		return $this->condition;
	}

	public function getJoins()
	{
		return $this->joins;
	}

	public function getPrefix()
	{
		return $this->prefix;
	}

	public function setPrefix($prefix)
	{
		$this->prefix           = $prefix;
		$this->selfColumns      = $this->getSelfColumns();
		$this->availableColumns = $this->selfColumns;

		return $this;
	}

	public function getColumns()
	{
		return $this->columns;
	}

	public function setColumns(array $columns)
	{
		$this->columns = $columns;

		return $this;
	}

	public function getSupportedFields()
	{
		return array_keys($this->availableColumns);
	}

	public function getSelfColumns()
	{
		$columns     = array();
		$selfColumns = $this->table->getColumns();

		foreach($selfColumns as $column => $attr)
		{
			$alias = $this->prefix !== null ? $this->prefix . ucfirst($column) : $column;
			$value = '`' . $this->table->getAlias() . '`.`' . $column . '`';

			$columns[$alias] = $value;
		}

		return $columns;
	}

	public function getAllColumns()
	{
		return $this->availableColumns;
	}

	public function getSelectedColumns()
	{
		$selectedColumns = array();
		$columns         = $this->table->getColumns();

		foreach($this->columns as $column)
		{
			if($this->prefix !== null && isset($columns[$column]))
			{
				$column = $this->prefix . ucfirst($column);
			}

			if(isset($this->availableColumns[$column]))
			{
				$selectedColumns[$column] = $this->availableColumns[$column];
			}
		}

		return $selectedColumns;
	}

	public function getAllSelectedColumns()
	{
		$selectedColumns = $this->getSelectedColumns();

		foreach($this->joins as $join)
		{
			$selectedColumns = array_merge($selectedColumns, $join->getTable()->getLastSelect()->getAllSelectedColumns());
		}

		return $selectedColumns;
	}

	public function getResultSet($startIndex = 0, $count = 32, $sortBy = null, $sortOrder = null, $filterBy = null, $filterOp = null, $filterValue = null, $updatedSince = null, $mode = 0, $class = null, array $args = array())
	{
		$start     = $startIndex !== null ? (integer) $startIndex : 0;
		$count     = $count      !== null ? (integer) $count      : 32;
		$sortBy    = $sortBy     !== null && isset($this->availableColumns[$sortBy]) ? $sortBy : current($this->availableColumns);
		$sortOrder = $sortOrder  !== null ? (strcasecmp($sortOrder, 'ascending') == 0 ? 'ASC' : 'DESC') : 'DESC';

		if(isset($this->availableColumns[$filterBy]))
		{
			switch($filterOp)
			{
				case 'contains':

					$this->where($filterBy, 'LIKE', '%' . $filterValue . '%');

					break;

				case 'equals':

					$this->where($filterBy, '=', $filterValue);

					break;

				case 'startsWith':

					$this->where($filterBy, 'LIKE', $filterValue . '%');

					break;

				case 'present':

					$this->where($filterBy, 'IS NOT', 'NULL', 'AND');
					$this->where($filterBy, 'NOT LIKE', '');

					break;
			}
		}

		if($updatedSince !== null)
		{
			// search datetime field
			$dateColumn = $this->table->getFirstColumnWithType(self::TYPE_DATETIME);

			if($dateColumn !== null)
			{
				$datetime = new DateTime($updatedSince);

				$this->where($dateColumn, '>', $datetime->format(PSX_Time::SQL));
			}
		}

		$this->limit($start, $count);

		$totalResults = $this->getTotalResults();
		$startIndex   = 0;
		$entries      = $this->getAll($mode, $class, $args);

		$resultSet = new PSX_Data_ResultSet($totalResults, $startIndex, $count, $entries);

		return $resultSet;
	}

	public function getAll($mode = 0, $class = null, array $args = array())
	{
		if($mode === PSX_Sql::FETCH_OBJECT && $class === null && $args === array())
		{
			$class = $this->getDefaultRecordClass();
			$args  = $this->getDefaultRecordArgs();
		}

		return $this->sql->getAll($this->buildQuery(), $this->condition->getValues(), $mode, $class, $args);
	}

	public function getRow($mode = 0, $class = null, array $args = array())
	{
		if($mode === PSX_Sql::FETCH_OBJECT && $class === null && $args === array())
		{
			$class = $this->getDefaultRecordClass();
			$args  = $this->getDefaultRecordArgs();
		}

		$this->limit(1);

		return $this->sql->getRow($this->buildQuery(), $this->condition->getValues(), $mode, $class, $args);
	}

	public function getCol()
	{
		return $this->sql->getCol($this->buildQuery(), $this->condition->getValues());
	}

	public function getField()
	{
		$this->limit(1);

		return $this->sql->getField($this->buildQuery(), $this->condition->getValues());
	}

	public function getTotalResults()
	{
		return (integer) $this->sql->getField($this->buildCountQuery(), $this->condition->getValues());
	}

	public function buildJoins()
	{
		$sql = '';

		foreach($this->joins as $join)
		{
			$fk    = $join->getForeignKey();
			$table = $join->getTable();
			$cardi = $join->getCardinality();

			$sql.= $join->getType() . ' JOIN `' . $table->getName() . '` AS `' . $table->getAlias() . '` ON ';

			if($cardi[0] == '1')
			{
				$sql.= '`' . $this->table->getAlias() . '`.`' . $this->table->getPrimaryKey() . '` = ';
			}
			else if($cardi[0] == 'n')
			{
				$fk  = $fk === null ? $this->getForeignKeyByTable($this->table->getConnections(), $table->getName()) : $fk;
				$sql.= '`' . $this->table->getAlias() . '`.`' . $fk . '` = ';
			}

			if($cardi[1] == '1')
			{
				$sql.= '`' . $table->getAlias() . '`.`' . $table->getPrimaryKey() . '`';
			}
			else if($cardi[1] == 'n')
			{
				$fk  = $fk === null ? $this->getForeignKeyByTable($table->getConnections(), $this->table->getName()) : $fk;
				$sql.= '`' . $table->getAlias() . '`.`' . $fk . '`';
			}

			$sql.= ' ' . $table->getLastSelect()->buildJoins() . ' ';
		}

		return $sql;
	}

	protected function buildQuery()
	{
		$selectedColumns = $this->getAllSelectedColumns();

		if(empty($selectedColumns))
		{
			throw new PSX_Sql_Table_Exception('No valid columns selected');
		}

		// select
		$sql = 'SELECT ';
		$i   = 0;
		$len = count($selectedColumns) - 1;

		foreach($selectedColumns as $alias => $column)
		{
			$sql.= $column . ' AS `' . $alias . '`' . ($i < $len ? ',' : '');

			$i++;
		}

		$sql.= ' FROM `' . $this->table->getName() . '` AS `' . $this->table->getAlias() . '` ' . $this->buildJoins();

		// where
		if($this->condition->hasCondition())
		{
			$sql.= $this->condition->getStatment();
		}

		// group by
		if($this->groupBy !== null)
		{
			$sql.= ' GROUP BY ' . $this->groupBy;
		}

		// order
		if($this->sortBy !== null)
		{
			$sql.= ' ORDER BY ' . $this->sortBy . ' ' . $this->sortOrder;
		}

		// limit
		if($this->start !== null)
		{
			$sql.= ' LIMIT ' . $this->start . ', ' . $this->count;
		}

		return $sql;
	}

	protected function buildCountQuery()
	{
		// select
		$sql = 'SELECT COUNT(*) FROM `' . $this->table->getName() . '` AS `' . $this->table->getAlias() . '` ' . $this->buildJoins();

		// condition
		if($this->condition->hasCondition())
		{
			$sql.= $this->condition->getStatment();
		}

		return $sql;
	}

	protected function getForeignKeyByTable(array $connections, $foreignTable)
	{
		foreach($connections as $column => $table)
		{
			if($table == $foreignTable)
			{
				return $column;
			}
		}

		throw new PSX_Sql_Table_Exception($foreignTable . ' is not connected to ' . $this->table->getAlias());
	}

	protected function getDefaultRecordClass()
	{
		return substr(get_class($this->table), 0, -6);
	}

	protected function getDefaultRecordArgs()
	{
		return array($this->table);
	}
}

