<?php
/*
 *  $Id: TableAbstract.php 371 2011-11-20 18:36:52Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Sql_TableAbstract
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Sql
 * @version    $Revision: 371 $
 */
abstract class PSX_Sql_TableAbstract implements PSX_Sql_TableInterface
{
	private static $aliasId = 0;

	protected $alias;
	protected $sql;
	protected $select;

	public function __construct(PSX_Sql $sql)
	{
		$this->sql = $sql;

		$this->setAlias(self::$aliasId++);
	}

	public function getAlias()
	{
		return $this->alias;
	}

	public function setAlias($alias)
	{
		$this->alias = $alias;
	}

	public function getSql()
	{
		return $this->sql;
	}

	public function getDisplayName()
	{
		$name = $this->getName();
		$pos  = strrpos($name, '_');

		return $pos !== false ? substr($name, strrpos($name, '_') + 1) : $name;
	}

	public function getPrimaryKey()
	{
		return $this->getFirstColumnWithAttr(self::PRIMARY_KEY);
	}

	public function getFirstColumnWithAttr($searchAttr)
	{
		$columns = $this->getColumns();

		foreach($columns as $column => $attr)
		{
			if($attr & $searchAttr)
			{
				return $column;
			}
		}

		return null;
	}

	public function getFirstColumnWithType($searchType)
	{
		$columns = $this->getColumns();

		foreach($columns as $column => $attr)
		{
			if(((($attr >> 20) & 0xFF) << 20) === $searchType)
			{
				return $column;
			}
		}

		return null;
	}

	public function getValidColumns(array $columns)
	{
		return array_intersect($columns, array_keys($this->getColumns()));
	}

	public function select(array $columns = array(), $prefix = null)
	{
		$this->select = new PSX_Sql_Table_Select($this, $prefix);

		if(in_array('*', $columns))
		{
			$this->select->setColumns($this->getColumns());
		}
		else
		{
			$this->select->setColumns($columns);
		}

		return $this->select;
	}

	public function getLastSelect()
	{
		return $this->select;
	}

	public function getRecord($id = null)
	{
		$class = $this->getDefaultRecordClass();

		if($id !== null)
		{
			$fields = implode(', ', array_map('PSX_Sql::helpQuote', array_keys($this->getColumns())));
			$sql    = 'SELECT ' . $fields . ' FROM `' . $this->getName() . '` WHERE `' . $this->getPrimaryKey() . '` = ?';
			$record = $this->sql->getRow($sql, array($id), PSX_Sql::FETCH_OBJECT, $class, $this->getDefaultRecordArgs());

			if(empty($record))
			{
				throw new PSX_Sql_Exception('Invalid record id');
			}
		}
		else
		{
			$ref = new ReflectionClass($class);

			$record = $ref->newInstanceArgs($this->getDefaultRecordArgs());
		}

		return $record;
	}

	public function getAll(array $fields, PSX_Sql_Condition $condition = null, $sortBy = null, $sortOrder = 0, $startIndex = null, $count = 32)
	{
		$fields = $this->getValidColumns($fields);

		return $this->sql->select($this->getName(), $fields, $condition, PSX_Sql::SELECT_ALL, $sortBy, $sortOrder, $startIndex, $count);
	}

	public function getRow(array $fields, PSX_Sql_Condition $condition = null, $sortBy = null, $sortOrder = 0)
	{
		$fields = $this->getValidColumns($fields);

		return $this->sql->select($this->getName(), $fields, $condition, PSX_Sql::SELECT_ROW, $sortBy, $sortOrder);
	}

	public function getCol(array $fields, PSX_Sql_Condition $condition = null, $sortBy = null, $sortOrder = 0, $startIndex = null, $count = 32)
	{
		$fields = $this->getValidColumns($fields);

		return $this->sql->select($this->getName(), $fields, $condition, PSX_Sql::SELECT_COL, $sortBy, $sortOrder, $startIndex, $count);
	}

	public function getField($field, PSX_Sql_Condition $condition = null, $sortBy = null, $sortOrder = 0)
	{
		$fields = $this->getValidColumns(array($field));

		return $this->sql->select($this->getName(), $fields, $condition, PSX_Sql::SELECT_FIELD, $sortBy, $sortOrder);
	}

	public function count(PSX_Sql_Condition $condition = null)
	{
		return $this->sql->count($this->getName(), $condition);
	}

	public function insert(array $params, $modifier = 0)
	{
		// check whether a needed value is not set
		$columns = $this->getColumns();

		foreach($columns as $column => $attr)
		{
			if(!$attr & self::IS_NULL && !isset($params[$column]))
			{
				throw new PSX_Sql_Table_Exception('Missing column ' . $column);
			}
		}

		return $this->sql->insert($this->getName(), $params, $modifier);
	}

	public function update(array $params, PSX_Sql_Condition $condition = null, $modifier = 0)
	{
		return $this->sql->update($this->getName(), $params, $condition, $modifier);
	}

	public function replace(array $params, PSX_Sql_Condition $condition = null, $modifier = 0)
	{
		return $this->sql->replace($this->getName(), $params, $condition, $modifier);
	}

	public function delete(PSX_Sql_Condition $condition = null, $modifier = 0)
	{
		return $this->sql->delete($this->getName(), $condition, $modifier);
	}

	protected function getDefaultRecordClass()
	{
		return substr(get_class($this), 0, -6);
	}

	protected function getDefaultRecordArgs()
	{
		return array($this);
	}
}


