<?php
/*
 *  $Id: Url.php 364 2011-11-13 23:53:40Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Url
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Url
 * @version    $Revision: 364 $
 */
class PSX_Url
{
	private $scheme;
	private $host;
	private $port;
	private $user;
	private $pass;
	private $path;
	private $query;
	private $fragment;

	public function __construct($url)
	{
		$filter = new PSX_Filter_Url();

		if($filter->apply($url) !== false)
		{
			$info = parse_url($url);

			$scheme   = isset($info['scheme'])   ? $info['scheme']       : null;
			$host     = isset($info['host'])     ? $info['host']         : null;
			$port     = isset($info['port'])     ? intval($info['port']) : null;
			$user     = isset($info['user'])     ? $info['user']         : null;
			$pass     = isset($info['pass'])     ? $info['pass']         : null;
			$path     = isset($info['path'])     ? $info['path']         : null;
			$fragment = isset($info['fragment']) ? $info['fragment']     : null;

			$query = array();

			if(isset($info['query']))
			{
				parse_str($info['query'], $query);
			}


			$this->setScheme($scheme);
			$this->setHost($host);
			$this->setPort($port);
			$this->setUser($user);
			$this->setPass($pass);
			$this->setPath($path);
			$this->setQuery($query);
			$this->setFragment($fragment);
		}
		else
		{
			throw new PSX_Url_Exception('Invalid url format');
		}
	}

	public function getScheme()
	{
		return $this->scheme;
	}

	public function getHost()
	{
		return $this->host;
	}

	public function getPort()
	{
		return $this->port;
	}

	public function getUser()
	{
		return $this->user;
	}

	public function getPass()
	{
		return $this->pass;
	}

	public function getPath()
	{
		return $this->path;
	}

	public function getQuery()
	{
		return $this->query;
	}

	public function getFragment()
	{
		return $this->fragment;
	}

	public function setScheme($scheme)
	{
		$this->scheme = $scheme;
	}

	public function setHost($host)
	{
		$this->host = $host;
	}

	public function setPort($port)
	{
		$this->port = $port;
	}

	public function setUser($user)
	{
		$this->user = $user;
	}

	public function setPass($pass)
	{
		$this->pass = $pass;
	}

	public function setPath($path)
	{
		$this->path = $path;
	}

	public function setQuery(array $query)
	{
		$this->query = $query;
	}

	public function setFragment($fragment)
	{
		$this->fragment = $fragment;
	}

	public function getContent()
	{
		if(in_array($this->getScheme(), stream_get_wrappers()))
		{
			return file_get_contents($this->getUrl());
		}
		else
		{
			throw new PSX_Url_Exception('Scheme not supported');
		}
	}

	public function getUrl()
	{
		$url = $this->scheme . '://';

		if(!empty($this->user) && !empty($this->pass))
		{
			$url.= $this->user . ':' . $this->pass . '@';
		}

		if(!empty($this->host))
		{
			$url.= $this->host;
		}
		else
		{
			throw new psx_filter_exception('Invalid host');
		}

		if(!empty($this->port) && $this->port != 80 && $this->port != 443)
		{
			$url.= ':' . $this->port;
		}

		if(!empty($this->path))
		{
			$url.= '/' . ltrim($this->path, '/');
		}

		if(!empty($this->query))
		{
			$url.= '?' . http_build_query($this->query, '', '&');
		}

		if(!empty($this->fragment))
		{
			$url.= '#' . $this->fragment;
		}

		return $url;
	}

	public function getParams()
	{
		return $this->query;
	}

	public function addParams(array $params)
	{
		foreach($params as $k => $v)
		{
			$this->addParam($k, $v);
		}
	}

	public function getParam($key)
	{
		return isset($this->query[$key]) ? $this->query[$key] : null;
	}

	public function addParam($key, $value, $replace = false)
	{
		if($replace === false)
		{
			if(!isset($this->query[$key]))
			{
				$this->query[$key] = $value;
			}
		}
		else
		{
			$this->query[$key] = $value;
		}
	}

	public function deleteParam($key)
	{
		if(isset($this->query[$key]))
		{
			unset($this->query[$key]);
		}
	}

	public function issetParam($key)
	{
		return isset($this->query[$key]);
	}

	public function __toString()
	{
		return $this->getUrl();
	}

	public static function encode($encode)
	{
		return urlencode($encode);
	}

	public static function decode($decode)
	{
		return urldecode($decode);
	}
}

