<?php
/*
 *  $Id: index.php 435 2011-12-03 18:16:10Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

require_once('../library/PSX/Config.php');
require_once('../library/PSX/Bootstrap.php');

$config    = new PSX_Config('../configuration.php');
$bootstrap = new PSX_Bootstrap($config);


ob_start('responseProcess');

try
{
	// load core libraries
	$template = new PSX_Template($config);
	$loader   = new PSX_Loader($config, $template);


	// add routes
	$loader->addRoute('/.well-known/host-meta', 'api/meta/host');


	// prolog modules
	// $loader->load('bar/foo');


	// load module
	$module = loadModule($config, $loader);


	// epilog modules
	// $loader->load('foo/bar');


	$content = ob_get_contents();


	// build response
	$response = buildResponse($config, $template, $content);


	// proccess response
	if($module !== null)
	{
		$module->processResponse($response);
	}
}
catch(PSX_Html_Exception $e)
{
	if($config['psx_debug'] === true)
	{
		$debug = 'Version: ' . $config['psx_version'] . "\n";
		$debug.= 'Message: ' . $e->getMessage() . "\n";
		$debug.= 'Code: ' . $e->getCode() . "\n";
		$debug.= 'File: ' . $e->getFile() . "\n";
		$debug.= 'Line: ' . $e->getLine() . "\n";
		$debug.= 'Exception: ' . get_class($e) . "\n";
		$debug.= $e->getTraceAsString();

		$template->assign('debug', $debug);
	}


	$template->assign('message', $e->getMessage());

	$template->set('system/exception.tpl');


	// build response
	$response = buildResponse($config, $template);
}
catch(Exception $e)
{
	$code = isset(PSX_Http::$codes[$e->getCode()]) ? $e->getCode() : 500;

	header('HTTP/1.1 ' . $code . ' ' . PSX_Http::$codes[$code]);
	header('Content-Type: text/plain');

	$response = $e->getMessage();

	if($config['psx_debug'] === true)
	{
		$response.= "\n\n" . $e->getTraceAsString();
	}
}

ob_end_clean();

echo $response;


/**
 * import
 *
 * @param string $x
 * @return void
 */
function import($x)
{
	global $loader;

	$loader->load($x);
}


/**
 * buildResponse
 *
 * @param PSX_Config $config
 * @param PSX_Template $template
 * @param string $content
 * @return string
 */
function buildResponse(PSX_Config $config, PSX_Template $template, $content = null)
{
	if(empty($content))
	{
		if($template->load() === false)
		{
			throw new PSX_Exception('Could not load template');
		}


		if(!($response = $template->transform()))
		{
			throw new PSX_Exception('Error while transforming template');
		}


		$acceptEncoding = PSX_Config::getRequestHeader('accept-encoding');

		if($config['psx_gzip'] === true && strpos($acceptEncoding, 'gzip') !== false)
		{
			header('Content-Encoding: gzip');

			$response = gzencode($response, 9);
		}
	}
	else
	{
		$response = $content;
	}

	return $response;
}


/**
 * responseProcess
 *
 * Callback function wich is called by the ob_start() function. This function
 * handles errors wich are not cought by the ErrorException handler.
 *
 * @return string
 */
function responseProcess($content)
{
	$lastError = error_get_last();

	if($lastError)
	{
		header('HTTP/1.1 500 Internal Server Error');
		header('Content-Type: text/plain');

		return $lastError['message'] . ' in ' . $lastError['file'] . ' on line ' . $lastError['line'] . "\n";
	}

	return $content;
}


/**
 * loadModule
 *
 * Loads the requested module depending on the psx_module_input field from the
 * config
 *
 * @return void
 */
function loadModule(PSX_Config $config, PSX_Loader $loader)
{
	$default = $config['psx_module_default'];
	$input   = $config['psx_module_input'];
	$length  = $config['psx_module_input_length'];

	if(!empty($input))
	{
		$x = $input;
	}
	else
	{
		$x = $default;
	}

	if(strpos($x, '..') !== false)
	{
		throw new PSX_Exception('Invalid signs in input');
	}

	if($length != 0)
	{
		if(strlen($x) > $length)
		{
			header('HTTP/1.1 414 Request-URI Too Long');

			throw new PSX_Exception('Max length of input is ' . $length);
		}
	}

	return $loader->load($x);
}


